# Example 3b using R

# set width of output
options("width"=120)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do linear combinations in gls

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means in gls


################################################################################
#####               BEGIN EXAMPLE 3b DATA MANIUPLATION                     #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/21_PSQF7375_Longitudinal/PSQF7375_Longitudinal_Example3b/"
filename = "SAS_Chapter3b.sas7bdat"
setwd(dir=filesave)

# Import chapter 3 six-occasion stacked data with labels
Example3b = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example3b = as.data.frame(Example3b)
# Labels can be used with gls, so I will use them here
# Sort data by PersonID (needed for correct RCOV matrix)
Example3b = sort_asc(Example3b,PersonID,session)
# Make new variable for session with reference=6 to match other programs
Example3b$session6=relevel(factor(Example3b$session), ref=6)

################################################################################
#####                   BEGIN EXAMPLE 3b MODELS                            #####
################################################################################

# Open output text file
txtStart(file=paste0(filesave,"R_Example3b_Output.txt"))

print("Chapter 3b Example: Means by session for RT outcome")
describeBy(x=Example3b$rt, digits=2, group=Example3b$session)  
txtComment(" ") # insert blank space in output


print("Eq 3.10: Between-Person Independent ANOVA via R GLS (VC R matrix, =LM)")
ANOVA_BP = gls(data=Example3b, method="REML",
               model=rt~1+factor(session6), 
               correlation=NULL) # VC R matrix (no correlation)
print("Show results with total leftover variance")
summary(ANOVA_BP); summary(ANOVA_BP)$sigma^2        
print("Session means, pairwise mean differences, and and omnibus F-test")
emmeans(ref_grid(ANOVA_BP), pairwise~(session6), adjust="none"); anova(ANOVA_BP) 
txtComment(" ") # insert blank space in output


print("Eq 3.10: Univariate Repeated Measures ANOVA via R GLS (CS R matrix)")
ANOVA_UnivWP = gls(data=Example3b, method="REML",
                   model=rt~1+factor(session6), 
                   correlation=corCompSymm(form=~1|PersonID)) 
print("Show results using incorrect DDF, with total leftover variance")
summary(ANOVA_UnivWP); summary(ANOVA_UnivWP)$sigma^2 
print("Show R and RCORR matrices for first person in data")
getVarCov(ANOVA_UnivWP, individual="101");
corMatrix(ANOVA_UnivWP$modelStruct$corStruct)[[5]]   
print("Session means, pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(ANOVA_UnivWP), pairwise~session6, adjust="none"); anova(ANOVA_UnivWP)
print("Show likelihood ratio test comparing model fit: BP vs. Univ WP")
anova(ANOVA_UnivWP,ANOVA_BP)  
txtComment(" ") # insert blank space in output


print("Eq 3.10: Multivariate Repeated Measures ANOVA via R GLS (UN R matrix)")
ANOVA_MultivWP = gls(data=Example3b, method="REML",
                     model=rt~1+factor(session6), 
                     correlation=corSymm(form=~as.numeric(session6)|PersonID),
                     weights=varIdent(form=~1|session6)) 
print("Show results using incorrect DDF with total leftover variance")
print("Total variance per occasion is created using SD multiplier")
summary(ANOVA_MultivWP); summary(ANOVA_MultivWP)$sigma^2 
print("Show R and RCORR matrices for first person in the data") 
getVarCov(ANOVA_MultivWP, individual="101");  
corMatrix(ANOVA_MultivWP$modelStruct$corStruct)[[5]] 
print("Session means, pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(ANOVA_MultivWP), pairwise~session6, adjust="none") # tried mode="df.error"
print("Error because of Satterthwaite DDF, so had to use lsmeans with incorrect DDF")
lsmeans(ANOVA_MultivWP, "session6", mode="df.error")
joint_tests(ANOVA_MultivWP, mode="df.error")
print("Show likelihood ratio test comparing model fit: Univ WP vs. Multiv WP")
anova(ANOVA_MultivWP,ANOVA_UnivWP)  
txtComment(" ") # insert blank space in output


# Close output text file
txtStop()
